package edsminer.hqui;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.map.Map;
import edsminer.deploymentAdvisor.DeploymentAdvisor;
import edsminer.general.edsTran;
import edsminer.strategyAnalysis.StrategyAnalysis;
import edsminer.simulationAgent.SimulationAgent;

//import edsminer.repository.Repository;
import java.util.Random;

/**
 * Modified to execute the following use cases:
 * 		StrategyAnalysis
 * 		SimulationAgent
 * 		DeploymentAdvisor
 * 		MapView
 * If the 3rd argument is set, the main method also tries to inject abnormal transactions
 * based on predefined anomaly rate.
 * 
 * Modified from Hqui class to allow for different user profiles: e.g. Novice and Expert
 * 
 * @author Eric Yuan
 *
 */
public class HquiPro {
	
//	private static final String mapName = "rmi://EDSMine/Map";
//	private static final String repositoryName = "rmi://EDSMine/Repository";
	private static final String mapName = "rmi://EDSMine/MapView";
	private static final String deploymentadvisorName = "rmi://EDSMine/DeploymentAdvisor";
	private static final String simulationagentName = "rmi://EDSMine/SimulationAgent";
	private static final String simulationagentanomalyName = "rmi://EDSMine/SimulationAgentAnomaly";
	private static final String strategyanalysisName = "rmi://EDSMine/StrategyAnalysis";
	private static final String strategyanalysisanomalyName = "rmi://EDSMine/StrategyAnalysisAnomaly";
	
	private static final double ANOMALY_INTERVAL_003 = 3.0;			// = 0.27% Apriori Anomaly Rate (3-sigma)
	private static final double ANOMALY_INTERVAL_005 = 2.807034;	// = 0.50% Apriori Anomaly Rate
	private static final double ANOMALY_INTERVAL_010 = 2.575829;	// = 1.00% Apriori Anomaly Rate
	private static final double ANOMALY_INTERVAL_015 = 2.432379;	// = 1.50% Apriori Anomaly Rate
	private static final double ANOMALY_INTERVAL_020 = 2.326348;	// = 2.00% Apriori Anomaly Rate
	private static final double ANOMALY_INTERVAL_025 = 2.241403;	// = 2.50% Apriori Anomaly Rate
	private static final double ANOMALY_INTERVAL_035 = 2.108358;	// ~ 3.50% Apriori Anomaly Rate
	private static final double ANOMALY_INTERVAL_050 = 1.959964;	// = 5.00% Apriori Anomaly Rate
	
	private static boolean insertAnomaly = false;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		if (args.length < 3) {
			System.out.println("Usage: HquiPro <registryHost> <userID> <userProfile> [ 1 ]");
			System.exit(1);
		}
		
		if (args.length > 3 && args[3].equals("1")) {
			System.out.println("HQUI will add anomaly transactions.");
			insertAnomaly = true;
		}

		// Expert user: shorter thinking time, can do strategy analysis
		boolean isExpert = args[2].equals(Tran.USER_EXPERT);
		// Novice user: longer thinking time, cannot do strategy analysis
		boolean isNovice = args[2].equals(Tran.USER_NOVICE);
		if (! isExpert && ! isNovice) {
			System.out.println("HquiPro: Invalid user profile " + args[2]);
			System.exit(1);
		}
		
		try{
			/* Get some controller parameters */
			codeController cont = new codeController();
			String sessionID = args[1];
			sessionID = sessionID.trim();
			
			/* Get a hold of the RMI components */
			Registry registry = LocateRegistry.getRegistry(args[0]);
			Map map = (Map) registry.lookup(mapName);
			DeploymentAdvisor deploymentAdvisor = null;
			SimulationAgent simulationAgent = null;
			SimulationAgent simulationAgentAnomaly = null;
			StrategyAnalysis strategyAnalysis = null;
			StrategyAnalysis strategyAnalysisAnomaly = null;
			
			if (cont.sooper()){
				deploymentAdvisor = (DeploymentAdvisor) registry.lookup(deploymentadvisorName);
				simulationAgent = (SimulationAgent) registry.lookup(simulationagentName);
				simulationAgentAnomaly = (SimulationAgent) registry.lookup(simulationagentanomalyName);
				strategyAnalysis = (StrategyAnalysis) registry.lookup(strategyanalysisName);
				strategyAnalysisAnomaly = (StrategyAnalysis) registry.lookup(strategyanalysisanomalyName);
			}

			/* Create a transaction object */
			edsTran tran = new edsTran();
			tran.setUserProfile(args[2]);
			
			/* Set apriori error rate */
			double anomalyInterval = ANOMALY_INTERVAL_003;
//			double anomalyInterval = ANOMALY_INTERVAL_005;
//			double anomalyInterval = ANOMALY_INTERVAL_010;
//			double anomalyInterval = ANOMALY_INTERVAL_015;
//			double anomalyInterval = ANOMALY_INTERVAL_020;
//			double anomalyInterval = ANOMALY_INTERVAL_025;
//			double anomalyInterval = ANOMALY_INTERVAL_035;
//			double anomalyInterval = ANOMALY_INTERVAL_050;
			
			/* Start the transaction loop */
			double meanThinkTime = isExpert ? 0.5 : 2.0;
			Random ran1 = new Random();
			Random ran2 = new Random();
			Random ran3 = new Random();
			int ranNum = 0, loops = 2500, cap = 2, sleepCap = 5, ranFixed = 0;
			double ranSleep = 0, ranSleep1 = 0, ranSleep2 = 0;
			if (cont.sooper()) {cap = 4;}
			for(int i = 0; i < loops; i++){
				ranNum = ran1.nextInt(cap);
				tran.setSeq(sessionID + "_" + i);
				
				// choose the sleep time
				ranSleep1 = ran2.nextGaussian() + meanThinkTime;
				ranSleep2 = ran3.nextGaussian() + meanThinkTime;
				ranSleep = Math.sqrt((ranSleep1 * ranSleep1) + (ranSleep2 * ranSleep2));
				//ranSleep = 0;	
				
				System.out.println("User thinking for " + ranSleep + " seconds.");
				try {Thread.sleep(((int) Math.round(ranSleep)) * 1000);
				} catch (Exception e) {
					System.err.println("HquiPro Sleep exception:");
					e.printStackTrace();
				}
				
				// Insert anomalous transaction, if desired
				double ranGauss1 = ran2.nextGaussian();
				double ranGauss2 = ran2.nextGaussian();
				if (insertAnomaly && (ranGauss1 > anomalyInterval || ranGauss1 < - anomalyInterval)) {
					System.out.println(i + " of " + loops + ": !!! ANOMALY --- Calling the StrategyAnalysisAnomaly component.");
					tran.setID("HQUItoSTRATEGYANALYSIS");
					strategyAnalysisAnomaly.executeTransaction(tran);
//				}else if (insertAnomaly && (ranGauss2 > anomalyInterval || ranGauss2 < - anomalyInterval)){
//					System.out.println(i + " of " + loops + ": !!! ANOMALY --- Calling the SimulationAgentAnomaly component.");
//					tran.setID("HQUItoSIMULATIONAGENT");
//					simulationAgentAnomaly.executeTransaction(tran);
				}else if (ranNum == 0){
					System.out.println(i + " of " + loops + ": Calling the Map component.");
					tran.setID("HQUItoMAP");
					map.executeTransaction(tran);
				}else if (ranNum == 1){
					System.out.println(i + " of " + loops + ": Calling the DeploymentAdvisor component.");
					tran.setID("HQUItoDEPLOYMENTADVISOR");
					deploymentAdvisor.executeTransaction(tran);
//					if (isExpert) {
//						System.out.println(i + " of " + loops + ": Calling the DeploymentAdvisor component.");
//						tran.setID("HQUItoDEPLOYMENTADVISOR");
//						deploymentAdvisor.executeTransaction(tran);
//					} else {
//						try {Thread.sleep(((int) Math.round(ranSleep)) * 1000);
//						} catch (Exception e) {
//							System.err.println("HquiPro Sleep exception:");
//							e.printStackTrace();
//						}
//					}
				}else if (ranNum == 2){
					System.out.println(i + " of " + loops + ": Calling the SimulationAgent component.");
					tran.setID("HQUItoSIMULATIONAGENT");
					simulationAgent.executeTransaction(tran);
				}else if (ranNum == 3){
//					System.out.println(i + " of " + loops + ": Calling the StrategyAnalysis component.");
//					tran.setID("HQUItoSTRATEGYANALYSIS");
//					strategyAnalysis.executeTransaction(tran);
					if (! isExpert) {
						System.out.println(i + " of " + loops + ": Calling the StrategyAnalysis component.");
						tran.setID("HQUItoSTRATEGYANALYSIS");
						strategyAnalysis.executeTransaction(tran);
					} else {
						try {Thread.sleep(((int) Math.round(ranSleep)) * 1000);
						} catch (Exception e) {
							System.err.println("HquiPro Sleep exception:");
							e.printStackTrace();
						}
					}
				}
			}
		} catch (Exception e){
			System.err.println("Hqui exception:");
			e.printStackTrace();
		}
	}
}
