package edsminer.hqui6;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import edsminer.map.Map;
import edsminer.repository.Repository;
import edsminer.general.codeController;
import edsminer.general.edsTran;
import edsminer.strategyAnalysis.StrategyAnalysis;
import edsminer.simulationAgent.SimulationAgent;

import java.util.Random;

public class Hqui6 {
	
	private static final String mapName = "rmi://EDSMine/Map";
	private static final String repositoryName = "rmi://EDSMine/Repository";
	private static final String simulationagentName = "rmi://EDSMine/SimulationAgent";
	private static final String strategyanalysisName = "rmi://EDSMine/StrategyAnalysis";
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		try{
			/* Get some controller parameters */
			codeController cont = new codeController();
			
			/* Get a hold of the RMI components */
			Registry registry = LocateRegistry.getRegistry(args[0]);
			Map map = (Map) registry.lookup(mapName);
			Repository repository = (Repository) registry.lookup(repositoryName);
			SimulationAgent simulationAgent = null;
			StrategyAnalysis strategyAnalysis = null;
			if (cont.sooper()){
				simulationAgent = (SimulationAgent) registry.lookup(simulationagentName);
				strategyAnalysis = (StrategyAnalysis) registry.lookup(strategyanalysisName);
			}

			/* Create a transaction object */
			edsTran tran = new edsTran();
			
			/* Start the transaction loop */
			Random ran = new Random();
			int ranNum = 0, ranSleep = 0, loops = 5000, cap = 2, sleepCap = 5;
			if (cont.sooper()) {cap = 4;}
			for(int i = 0; i < loops; i++){
				ranNum = ran.nextInt(cap);
				ranSleep = ran.nextInt(sleepCap);
				tran.setSeq("6_" + i);
				
				System.out.println("User thinking for " + ranSleep + " seconds.");
				try {Thread.sleep((ranSleep+2) * 1000);
				} catch (Exception e) {
					System.err.println("MapEngine Sleep exception:");
					e.printStackTrace();
				}
				
				ranSleep = ran.nextInt(sleepCap * 4);
				if (ranSleep == 0){
					System.out.println("User has left, will return in 5 minutes.");
					try {Thread.sleep(1000 * 60 * 5);
					} catch (Exception e) {
						System.err.println("Sleep exception:");
						e.printStackTrace();
					}
				}
				
				if (ranNum == 0){
					System.out.println(i + " of " + loops + ": Calling the Map component.");
					tran.setID("HQUI6toMAP");
					map.executeTransaction(tran);
				}else if (ranNum == 1){
					System.out.println(i + " of " + loops + ": Calling the Repository component.");
					tran.setID("HQUI6toREPOSITORY");
					repository.executeTransaction(tran);
				}else if (ranNum == 2){
					System.out.println(i + " of " + loops + ": Calling the SimulationAgent component.");
					tran.setID("HQUI6toSIMULATIONAGENT");
					simulationAgent.executeTransaction(tran);
				}else if (ranNum == 3){
					System.out.println(i + " of " + loops + ": Calling the StrategyAnalysis component.");
					tran.setID("HQUI6toSTRATEGYANALYSIS");
					strategyAnalysis.executeTransaction(tran);
				}
			}
		} catch (Exception e){
			System.err.println("HQUI6 exception:");
			e.printStackTrace();
		}
	}
}
