package edsminer.map;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Random;
// s
import java.util.Vector;
// e

import edsminer.clock.Clock;
import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.weather.Weather;
import edsminer.resourceManager.ResourceManager;
import edsminer.repository.Repository;

/**
 * A map engine that serves as the entry point of a top level transaction.
 * It calls other components as described in the use case.
 * The original MapViewEngine class is modified to be a "leaf" component that does not call other components.
 * 
 * @author Eric Yuan
 *
 */
public class MapViewEngine implements Map {
	private static codeController cont = new codeController();
	private static boolean setupSuccessful = true;
	private static Registry registry = null;
	private static Connection connect = null;
	private static PreparedStatement initStmt = null;
	private static PreparedStatement wrapStmt = null;
	private static final String weatherName = "rmi://EDSMine/Weather";
	private static final String repositoryName = "rmi://EDSMine/Repository";
	private Random ran = new Random(42);
	private static Weather weather = null;
	private static Repository repository = null;
	private int ranNum = 0;
	private Date initDate;
	private Date termDate;
	private Timestamp tmstpInit;
	private Timestamp tmstpTerm;
	private String tranID;
	private String seqID;
	private static int cap;
	private static int init_pos;
	private Vector<String> tran_vec = new Vector();
	private Vector<String> seq_vec = new Vector();
	private Vector<java.sql.Timestamp> tmstp_vec = new Vector();
	
	public MapViewEngine(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		/* Try to get a hold of components needed through RMI if it didn't work before */
		if (!setupSuccessful){
			try {
				if (cont.sooper()){
					if (weather == null) {weather = (Weather) registry.lookup(weatherName);}
					if (repository == null) {repository = (Repository) registry.lookup(repositoryName);}
				}
				setupSuccessful = true;
			} catch (Exception e) {
				System.err.println("MapViewEngine RMI lookup exception:");
				e.printStackTrace();
			}
		}
		
		/* Try to log the entry into the method */
		try {
			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			
			// set the value for tran_init_tmstp and tran_id
			// s
			initStmt.setLong(1, tmstpInit.getTime());
			// e
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
			
			// store off the values for return later on
			//tran_vec.add(tranID);
			//seq_vec.add(seqID);
			//tmstp_vec.add(tmstpInit);
		} catch (Exception e) {
			System.err.println("MapViewEngine LogEntry exception:");
			e.printStackTrace();
		}
		
		boolean done = false;
		
		while (!done){
			// even probabilities of calling weather, repository, or calling nothing
			ranNum = ran.nextInt(cap*2);
			if (ranNum == 0 || ranNum == 1){
				System.out.println("Calling the Repository component.");
				t.setID((T) "MAPtoREPOSITORY");
				seqID = (String) repository.executeTransaction(t);
			} else if (ranNum == 2 || ranNum == 3){
				System.out.println("Calling the Weather component.");
				t.setID((T) "MAPtoWEATHER");
				seqID = (String) weather.executeTransaction(t);
			} else if (ranNum == 4 || ranNum == 5) {
				/* Perform the work of the method here */
				System.out.println("MapViewEngine executing a transaction.");
				try {Thread.sleep(500);
				} catch (Exception e) {
					System.err.println("MapViewEngine Sleep exception:");
					e.printStackTrace();
				}
			} else if (ranNum == 6 || ranNum == 7) {
				done = true;
			}
		}
		
		/* Try to log the exit from the method */
		try {
			// set the exit timestamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			//init_pos = seq_vec.size() - 1;
			
			// find the most recent init values for this sequence id
			//while (!(seqID == seq_vec.elementAt(init_pos))){
			//	init_pos--;				
			//}
			
			// now update with the real original values
			//tranID = tran_vec.remove(init_pos);
			//seqID = seq_vec.remove(init_pos);
			//tmstpInit = tmstp_vec.remove(init_pos);
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			// s
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			// e
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
		} catch (Exception e) {
			System.err.println("MapViewEngine LogExit exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		cap = 3;
		if (cont.sooper()) {cap = 4;}
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/MapView";
			Map engine = new MapViewEngine();
			Map stub = (Map) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("MapViewEngine bound");
		} catch (Exception e) {
			System.err.println("MapViewEngine RMI registration exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			// s
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			// e
			
		} catch (Exception e) {
			System.err.println("MapViewEngine DB exception:");
			e.printStackTrace();
		}
		
		/* Try to get a hold of components needed through RMI */
		try {
			registry = LocateRegistry.getRegistry(args[0]);
			if (cont.sooper()){
				repository = (Repository) registry.lookup(repositoryName);
				weather = (Weather) registry.lookup(weatherName);
			}
		} catch (Exception e) {
			setupSuccessful = false;
			System.err.println("MapViewEngine RMI lookup exception:");
			e.printStackTrace();
		}
	}
}
