package edsminer.resourceManager;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Random;

import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.resourceManager.ResourceManager;
import edsminer.resourceMonitor.ResourceMonitor;

public class ResourceManagerEngineOld implements ResourceManager {
	private static codeController cont = new codeController();
	private static boolean setupSuccessful = true;
	private static Registry registry = null;
	private static Connection connect = null;
	private static PreparedStatement initStmt = null;
	private static PreparedStatement wrapStmt = null;
	// start sooper
	private static final String resourcemonitorName = "rmi://EDSMine/ResourceMonitor";
	private static ResourceMonitor resourceMonitor = null;
	// end sooper
	private Random ran = new Random(42);
	private int ranNum = 0;
	private Date initDate;
	private Date termDate;
	private Timestamp tmstpInit;
	private Timestamp tmstpTerm;
	private String tranID;
	private String seqID;
	private static int cap;
	
	public ResourceManagerEngineOld(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		/* Try to get a hold of components needed through RMI if it didn't work before */
		if (!setupSuccessful){
			try {
				// start sooper
				if (cont.sooper()){
					if (resourceMonitor == null) {resourceMonitor = (ResourceMonitor) registry.lookup(resourcemonitorName);}
				}// end sooper
				setupSuccessful = true;
			} catch (Exception e) {
				System.err.println("ResourceManager RMI lookup exception:");
				e.printStackTrace();
			}
		}
		
		/* Try to log the entry into the method */
		try {
			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			
			// set the value for tran_init_tmstp and tran_id
			initStmt.setLong(1, tmstpInit.getTime());
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine LogEntry exception:");
			e.printStackTrace();
		}
				
		/* Perform the work of the method here */
		System.out.println("ResourceManagerEngine executing a transaction.");
		try {Thread.sleep(500);
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine Sleep exception:");
			e.printStackTrace();
		}
		
		ranNum = ran.nextInt(cap*2);
		
		if (ranNum == 0) {
			System.out.println("Returning without calling other components.");
			// start update
		}else if (ranNum == 1 || ranNum == 2 || ranNum == 3) {
			System.out.println("Calling the ResourceMonitor component.");
			t.setID((T) "RESOURCEMANAGERtoRESOURCEMONITOR");
			resourceMonitor.executeTransaction(t);
		}
		
		/* Try to log the exit from the method */
		try {
			// set the exit timestamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine LogExit exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		cap = 1;
		// start sooper
		if (cont.sooper()) {cap = 2;}
		// end sooper
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/ResourceManager";
			ResourceManager engine = new ResourceManagerEngineOld();
			ResourceManager stub = (ResourceManager) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("ResourceManagerEngine bound");
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine RMI exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine DB exception:");
			e.printStackTrace();
		}
		
		/* Try to get a hold of components needed through RMI */
		try {
			registry = LocateRegistry.getRegistry(args[0]);
			// start sooper
			if (cont.sooper()){
				resourceMonitor = (ResourceMonitor) registry.lookup(resourcemonitorName);
			}// end sooper
		} catch (Exception e) {
			// start update
			setupSuccessful = false;
			// end update
			System.err.println("ResourceManager RMI lookup exception:");
			e.printStackTrace();
		}
	}
}
