package edsminer.simulationAgent;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Random;
import java.util.Vector;

import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.map.Map;
import edsminer.clock.Clock;
import edsminer.repository.Repository;
import edsminer.resourceMonitor.ResourceMonitor;
import edsminer.strategyAnalysis.StrategyAnalysis;

/**
 * Implements an anomalous transaction (Hijacked SimulationAgent component)
 * 
 * Thread-safe version.
 * 
 * @author Eric Yuan
 *
 */
public class SimulationAgentAnomalyEngineTS implements SimulationAgent {
	private static codeController cont = new codeController();
	private static boolean setupSuccessful = true;
	private static Registry registry = null;
//	private static Connection connect = null;
//	private static PreparedStatement initStmt = null;
//	private static PreparedStatement wrapStmt = null;
//	private Date initDate;
//	private Date termDate;
//	private Timestamp tmstpInit;
//	private Timestamp tmstpTerm;
//	private String tranID;
//	private String seqID;
	private static final String mapName = "rmi://EDSMine/Map";
	private static final String repositoryName = "rmi://EDSMine/Repository";
	private static final String resourceMonitorName = "rmi://EDSMine/ResourceMonitor";
	private static final String clockName = "rmi://EDSMine/Clock";
	//private static final String strategyanalysisName = "rmi://EDSMine/StrategyAnalysis";
	private Random ran = new Random(42);
	private static Map map = null;
	private static Repository repository = null;
	private static ResourceMonitor resourceMonitor = null;
	private static Clock clock = null;
	//private static StrategyAnalysis strategyAnalysis = null;
	private int ranNum = 0;
	private static int cap;
	private static int init_pos;
	private Vector<String> tran_vec = new Vector();
	private Vector<String> seq_vec = new Vector();
	private Vector<java.sql.Timestamp> tmstp_vec = new Vector();
	
	public SimulationAgentAnomalyEngineTS(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		Connection connect = null;
		PreparedStatement initStmt = null;
		PreparedStatement wrapStmt = null;
		Date initDate = null;
		Date termDate = null;
		Timestamp tmstpInit = null;
		Timestamp tmstpTerm = null;
		String tranID = null;
		String seqID = null;
		String userProfile = null;

		/* Try to get a hold of components needed through RMI if it didn't work before */
		if (!setupSuccessful){
			try {
				if (cont.sooper()){
					if (clock == null) {clock = (Clock) registry.lookup(clockName);}
					if (map == null) {map = (Map) registry.lookup(mapName);}
					if (repository == null) {repository = (Repository) registry.lookup(repositoryName);}
					if (resourceMonitor == null) {resourceMonitor = (ResourceMonitor) registry.lookup(resourceMonitorName);}
					//if (strategyAnalysis == null) {strategyAnalysis = (StrategyAnalysis) registry.lookup(strategyanalysisName);}
				}
				setupSuccessful = true;
			} catch (Exception e) {
				System.err.println("SimulationAgentAnomalyEngine RMI lookup exception:");
				e.printStackTrace();
			}
		}
		
		try {
			/* ----- Try to log the entry into the method ----- */
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");

			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			userProfile = (String) t.getUserProfile();
			
			// set the value for tran_init_tmstp and tran_id
			initStmt.setLong(1, tmstpInit.getTime());
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
			
			// store off the values for return later on
			//tran_vec.add(tranID);
			//seq_vec.add(seqID);
			//tmstp_vec.add(tmstpInit);
		
			/* ----- Perform the work of the method here ----- */
			// 3/4 chance we call clock
			ranNum = ran.nextInt(4);
			if (ranNum != 0){
				System.out.println("Calling the Clock component.");
				t.setID((T) "SIMULATIONAGENTtoCLOCK");
				seqID = (String) clock.executeTransaction(t);
			}
			
			// 50/50 chance we call repository
			ranNum = ran.nextInt(2);
			if (ranNum == 0){
				System.out.println("Calling the Repository component.");
				t.setID((T) "SIMULATIONAGENTtoREPOSITORY");
				seqID = (String) repository.executeTransaction(t);
			}
			
			// 3/4 chance we call map
			ranNum = ran.nextInt(4);
			if (ranNum == 0){
				System.out.println("Calling the Map component.");
				t.setID((T) "SIMULATIONAGENTtoMAP");
				seqID = (String) map.executeTransaction(t);
			}
		
		// 1/5 chance to call each of four components or return without calling
//		ranNum = ran.nextInt(cap);
//		if (ranNum == 0 || ranNum == 3 || ranNum == 4){
//			System.out.println("Calling the Map component.");
//			t.setID((T) "SIMULATIONAGENTtoMAP");
//			seqID = (String) map.executeTransaction(t);
//		}else if (ranNum == 1 || ranNum == 2) {
//			System.out.println("Returning without calling other components.");
//		}//else if (ranNum == 2 || ranNum == 3 || ranNum == 4) {
//			//System.out.println("Calling the StrategyAnalysis component.");
//			//t.setID((T) "SIMULATIONAGENTtoSTRATEGYANALYSIS");
//			//strategyAnalysis.executeTransaction(t);
//		//}
		
			// ANOMALOUS BEHAVIOR: Call ResourceMonitor
			ranNum = ran.nextInt(4);
			if (ranNum != 0) {
				System.out.println("ANOMALY --- SimulationAgent calling the ResourceMonitor component.");
				t.setID((T) "SIMULATIONAGENTtoRESOURCEMONITOR");
				resourceMonitor.executeTransaction(t);
			}
		
			try {Thread.sleep(500);
			} catch (Exception e) {
				System.err.println("SimulationAgentEngine Sleep exception:");
				e.printStackTrace();
			}
		
			// 50/50 chance we call repository
			ranNum = ran.nextInt(2);
			if (ranNum == 0){
				System.out.println("Calling the Repository component.");
				t.setID((T) "SIMULATIONAGENTtoREPOSITORY");
				seqID = (String) repository.executeTransaction(t);
			}
		
			// 3/4 chance we call clock
			ranNum = ran.nextInt(4);
			if (ranNum != 0){
				System.out.println("Calling the Clock component.");
				t.setID((T) "SIMULATIONAGENTtoCLOCK");
				seqID = (String) clock.executeTransaction(t);
			}
		
			/* ----- Try to log the exit from the method ----- */
			// set the exit timestamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			//init_pos = seq_vec.size() - 1;
			
			// find the most recent init values for this sequence id
			//while (!(seqID == seq_vec.elementAt(init_pos))){
			//	init_pos--;				
			//}
			
			// now update with the real original values
			//tranID = tran_vec.remove(init_pos);
			//seqID = seq_vec.remove(init_pos);
			//tmstpInit = tmstp_vec.remove(init_pos);
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
			
			connect.close();
		} catch (Exception e) {
			System.err.println("SimulationAgentAnomalyEngine server exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		cap = 0;
		if (cont.sooper()) {cap = 5;}
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/SimulationAgentAnomaly";
			SimulationAgent engine = new SimulationAgentAnomalyEngineTS();
			SimulationAgent stub = (SimulationAgent) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("SimulationAgentAnomalyEngine bound");
		} catch (Exception e) {
			System.err.println("SimulationAgentAnomalyEngine RMI registration exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
//			// Setup the connection with the DB
//			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
//			// create the prepared statement for the transaction_init table insertion
//			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
//			// create the prepared statement for the transaction_wrap table insertion
//			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			
		} catch (Exception e) {
			System.err.println("SimulationAgentAnomalyEngine DB exception:");
			e.printStackTrace();
		}
		
		/* Try to get a hold of components needed through RMI */
		try {
			registry = LocateRegistry.getRegistry(args[0]);
			if (cont.sooper()){
				clock = (Clock) registry.lookup(clockName);
				map = (Map) registry.lookup(mapName);
				repository = (Repository) registry.lookup(repositoryName);
				resourceMonitor = (ResourceMonitor) registry.lookup(resourceMonitorName);
				//strategyAnalysis = (StrategyAnalysis) registry.lookup(strategyanalysisName);
			}
		} catch (Exception e) {
			setupSuccessful = false;
			System.err.println("SimulationAgentAnomalyEngine RMI lookup exception:");
			e.printStackTrace();
		}
	}
}
